"""
Program  : SDMinP
Version  : 1.0
Date     : 16.03.2005
Purpose  : Adjustment of the multiple type I error (FWER)
Programer: Markus Obreiter
Contact  : m.obreiter@dkfz-heidelberg.de
Copyright (C) 2005 Deutsches Krebsforschungszentrum, DKFZ (German Cancer
Research Center), Division of Clinical
Epidemiology

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your option)
any later version. 
This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation,
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

A copy of the GNU Public License is attached to the program documentation.
"""

import sys, time
class CalculationRawP:

    def __init__(self, appCfgObject):
        self.appConfig      = appCfgObject
        self.testSideOption = self.appConfig.testCharacter
        self.calcRawPOption = self.appConfig.calcRawPFormula
        if self.calcRawPOption not in ['BECKER','GE_ET_AL']:
            sys.exit("No valid option entered for calcRawPvalueVector in module calculation.py")

        self.DEBUG = 0        
        
    def __calcRawPOriginal (self, tObs, lstTprm):
        """tObs: observed test statistic
           lstTprm: list of permutation test statistics
        """
        testCharacter = self.testSideOption
        
        lstT = lstTprm [:]
        noOfPermutations = len(lstT)
        if testCharacter == "RIGHT_SIDED":
            lstT.sort()
            lstT.reverse()
        elif testCharacter == "LEFT_SIDED":
            lstT.sort()        
        elif testCharacter == "TWO_SIDED":
            tObs = abs(tObs)
            for i in range(noOfPermutations):
                lstT[i] = abs(lstT[i])
            lstT.sort()
            lstT.reverse()
        else:
            sys.exit("no valid test character")

        count = 0
        for tPrm in lstT:
            if testCharacter in ["RIGHT_SIDED","TWO_SIDED"] and (tPrm >= tObs):
                count += 1
            elif testCharacter in ["LEFT_SIDED"] and (tPrm <= tObs):
                count += 1            
            else:
                break
            
        rawP = float(count)/noOfPermutations
        return rawP        

    

    def __calcRawP(self, lstTvalues, amount_permutations):
        """returns list of corresponding raw p values;
           option can be formula of 'BECKER' or 'GE_ET_AL'"""
    
        dictRawPvalues        = {} # keeps temporary result
        vectorPRawValuesFinal = [] # keeps the final results of p values
        ties = 0

        tvectorOriginal = lstTvalues[:] # keeps the original ingoing tvector                  
        tvectorSorted   = []            # keeps the sorted tvalues; is used to calculate the raw p values

        # -------------------------------------------------------------------------------------------------------------
        # check for one- or two-sided test; in case of two-sided, the t-values have to be made absolute
        
        if self.testSideOption in ['LEFT_SIDED', 'RIGHT_SIDED']:
            tvectorSorted   = tvectorOriginal[:]           
        elif self.testSideOption == 'TWO_SIDED':
            for tval in tvectorOriginal:
                tval = abs(tval)
                tvectorSorted.append(tval)                
        else:
            sys.exit("No valid option entered for calcRawPvalueVector in module calculation.py")
            
        # ------------------------------------------------------------------------------------------------------------- 
        
        if self.testSideOption in ['TWO_SIDED', 'RIGHT_SIDED']:
            #start with biggest element: descending
            tvectorSorted.sort()
            tvectorSorted.reverse()
        elif self.testSideOption == 'LEFT_SIDED':
            # start with smallest element: ascending
            tvectorSorted.sort()
                  
        # -------------------------------------------------------------------------------------------------------------
        # Calculate raw p value for each tvalue
        loopCount   = 0
        listLength = len(tvectorSorted)
        for position in range(listLength):
            loopCount += 1
            amountCount = -1
            pval = -1

            # 0) Preparation        
            tvalue = tvectorSorted[position]            
            if position <> 0:
                if tvectorSorted[position] == tvectorSorted[position - 1]:
                    # the values have already been calculated in the tie handling 
                    continue            

            # 1) check for ties
            Kj = 1
            tiePositionCount = position + 1
            if tiePositionCount <= (listLength -1):
                while tvectorSorted[position] == tvectorSorted[tiePositionCount]:                    
                    Kj = Kj + 1
                    tiePositionCount += 1
                    if tiePositionCount == listLength: break # end of list
                            
            if self.calcRawPOption == 'BECKER':
                pass
            elif self.calcRawPOption == 'GE_ET_AL':
                amountCount = amountCount + 1   # each element has to be compared with itself as well
            else:
                sys.exit("No valid option entered for calcRawPvalueVector in module calculation.py")
                
            if Kj > 1: # handling of ties                            
                #print "Ties detected for value:", str(tvalue)                 
                poslastj = position + Kj - 1      # range of last element with same T value as current one
                amountCount = poslastj            
            else: # no ties              
                amountCount = position            

            # 2) count amount
            if self.calcRawPOption == 'BECKER':
                # amountCount = amountCount, nothing happens
                pass
            elif self.calcRawPOption == 'GE_ET_AL':
                amountCount = amountCount + 1   # each element has to be compared with itself as well
            else:
                sys.exit("No valid option entered for calcRawPvalueVector in module calculation.py")
                
            # 3) calculate
            # float needed, otherwise '/' is interpreted as DIV                                       
            pval  = amountCount / float(amount_permutations)
            if not (0 <= pval)  and (pval <= 1):
                sys.exit("Exception: Wrong calculation in calculationRawP. Please contact the program developer.")
            dictRawPvalues[tvalue] = pval

        # -------------------------------------------------------------------------------------------------------------
        # Return pvector for ingoing unsorted tvector; each tvalue gets its respective pvalue assigned
        vectorPRawValuesFinal = []
        for tvalueOrig in tvectorOriginal:
            if self.testSideOption in ['LEFT_SIDED','RIGHT_SIDED']:
                vectorPRawValuesFinal.append(dictRawPvalues[tvalueOrig])                
            elif self.testSideOption == 'TWO_SIDED':
                vectorPRawValuesFinal.append(dictRawPvalues[abs(tvalueOrig)])
            else:
                sys.exit("No valid option entered for calcRawPvalueVector in module calculation.py")                
                
        return vectorPRawValuesFinal            

       
    def assignRawPvaluesPermutations (self, marker):               

        # security check
        if not (0 <= marker['pValRawOrg'])  and (marker['pValRawOrg'] <= 1):
            sys.exit("Invalid p-value for marker in line " + str(marker[ln]))

        #--------------------------------------
        lstTvalues = []
        amount_perm = len(marker['tVectorPrm'])
        
        if self.calcRawPOption == 'GE_ET_AL':
            # Observed test statistic is not included into the calculation
            marker['lstPvalRawPrm'] = self.__calcRawP(marker['tVectorPrm'], amount_perm)
            
        elif self.calcRawPOption == 'BECKER':
            # Observed test statistic is included into the calculation     
            lstTvalues.append(marker['tValOrg'])
            lstTvalues = lstTvalues + marker['tVectorPrm']
            rawPvalues = self.__calcRawP(lstTvalues, amount_perm)

            # a security check,
            if  rawPvalues[0] <> marker['pValRawOrg'] :
                print rawPvalues[0]
                print marker['pValRawOrg']
                sys.exit("error in p-value calculation for becker in marker line " + str(marker['ln']))
            marker['lstPvalRawPrm'] = rawPvalues[1:]                                    
                     
        
    def calcPValRawOrg (self, marker, lstTvalPermut):
        marker['pValRawOrg'] = self.__calcRawPOriginal(marker['tValOrg'], lstTvalPermut)
        
        
