"""
Program  : SDMinP
Version  : 1.0
Date     : 16.03.2005
Purpose  : Adjustment of the multiple type I error (FWER)
Programer: Markus Obreiter
Contact  : m.obreiter@dkfz-heidelberg.de
Copyright (C) 2005 Deutsches Krebsforschungszentrum, DKFZ (German Cancer
Research Center), Division of Clinical
Epidemiology

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your option)
any later version. 
This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation,
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

A copy of the GNU Public License is attached to the program documentation.
"""

import formatHelper, sys

class CalculationStepDownMinP:

    def __init__(self, appCfgObject):

        self.appConfig = appCfgObject
        self.DEBUG = 0
        
    def refreshMinQvector (self, qvector, pvector):
        """Input parameter is vector of p-values
           Return value: qvectornew
        """
        # calculate the values
        for b in range(len(pvector)):
            qvector[b] = min(qvector[b], pvector[b])
        return qvector


    def refreshMinAndSecMinQvector (self, qvectorMin, qvectorSec, lstPvalPermutations):

        for pos in range(len(lstPvalPermutations)):
            p    = lstPvalPermutations[pos]
            qMin = qvectorMin[pos]
            qSec = qvectorSec[pos]
            if p < qMin:
                qvectorMin[pos] = p
                qvectorSec[pos] = qMin                      
            elif p < qSec:
                qvectorSec[pos] = p
            else:
                pass
        

    def calcAdjustedPvalue(self, qvector, pvaluehypo):

        adjpval = -1
        count = 0
        B = len(qvector)

        for b in range(B):
            qval = qvector[b]
            if (qval <= pvaluehypo):
                count += 1
            else:
                pass

        adjpval = count / float(B) # float needed, otherwise / is interpreted as DIV 
        return adjpval

    def performStepDownProcedure(self, marker, qvectorMin, qvectorSec, amount_permutations, logfile):


        formatHlp  = formatHelper.FormatHelper()
        resultPadj = []

        vectRawPValues = marker['lstPvalRawPrm']
        pvalueRawOrg   = marker['pValRawOrg']        
        markerID = marker['id']

        # ----------------------------------------------------------------------------------------------------------
        # 2) calculate the successive minima
        if logfile:
            logfile.msgEoln ("")
            logfile.msgEoln("Test-ID : " + str(markerID))
            logfile.msg    ("Qmin in : ")
            logfile.msgEoln(formatHlp.formatFloatList(qvectorMin))
            if qvectorSec:
                logfile.msg ("Qsec in : ")                
                logfile.msgEoln(formatHlp.formatFloatList(qvectorSec))               

        if self.DEBUG: print "Refreshing q vector for marker line %i .." %marker['ln']
        if self.appConfig.calcGlobalPvalOption == 'SIMPLE':
            self.refreshMinQvector (qvectorMin, vectRawPValues)
        elif self.appConfig.calcGlobalPvalOption == 'EXTENDED':
            self.refreshMinAndSecMinQvector(qvectorMin, qvectorSec, vectRawPValues)
        else:
            sys.exit("No valid option for calculation of global p value.")
        if self.DEBUG: print "Refreshing q vector for marker line %i finished." %marker['ln']   

        if logfile:                
            logfile.msg ("PrmPval : ")
            logfile.msgEoln(formatHlp.formatFloatList(vectRawPValues))
            logfile.msg ("Qmin new: ")
            logfile.msgEoln(formatHlp.formatFloatList(qvectorMin))
            if qvectorSec:
                logfile.msg ("Qsec new: ")                
                logfile.msgEoln(formatHlp.formatFloatList(qvectorSec))           
        # ------------------------------------------------------------------------------------------------------------
        #3)(STEP 3 of the algorithm)
        # calculate the adjusted p-value for the original statistic            
        adjPval = self.calcAdjustedPvalue(qvectorMin, pvalueRawOrg)                      
        marker['adjPval'] = adjPval
        resultPadj.append(marker)

        if logfile:
            logfile.msgEoln("")        
            logfile.msg    ("original raw p  --> " + str(pvalueRawOrg))
            logfile.msgEoln("   (tvalue: " + str(marker['tValOrg']) + ")")
            logfile.msg    ("intermediate p  --> " + str(adjPval))
            logfile.msgEoln("   (adjusted, but without enforced monotonicity)")            
            logfile.msgEoln("")

        # ------------------------------------------------------------------------------------------------------------
        return qvectorMin, qvectorSec
                   

    # ----------------------------------------------------------------------------------------------------------------
    # ----------------------------------------------------------------------------------------------------------------


    def enforceMonotonicity (self, markerList, logFile):
        """markerList must be sorted from biggest raw p value to the smallest"""
        adjPvalLast = -1
        adjPvalCurrent  = -1
        loopcount = 0
        newlist = []

        # sort markerList from smallest raw P value to biggest
        #--> consider that it must be already sorted from max to min (so it comes from step down proc)!!!
        markerList.reverse() 

        for m in markerList:
            loopcount += 1
            if loopcount == 1:     
                adjPvalLast = m['adjPval']
            else:
                adjPvalCurrent = m['adjPval']
                adjPvalLast = max(adjPvalLast, adjPvalCurrent)
            
            m['adjPval_enforcedMonotonicity'] = adjPvalLast
            newlist.append(m)

        markerList = newlist[:]

        if logFile:
            logFile.msgEoln("")    
            logFile.msgEoln("Enforced Monotonicity")
            logFile.msgEoln("=====================")
            for m in markerList:
                logFile.msgEoln("")         
                logFile.msgEoln("Test-ID         --> " + str(m['id']))
                logFile.msgEoln("original raw p  --> " + str(m['pValRawOrg']))        
                logFile.msgEoln("intermediate p  --> " + str(m['adjPval']))
                logFile.msg    ("adjusted p      --> " + str(m['adjPval_enforcedMonotonicity']))
                logFile.msgEoln("    (enforced monotonicity)")

        return markerList

